# Base includes
import random
import math

# Game engine includes
import pygame
from pygame.locals import *
import ode

# PGU includes
from globalvars import *
from pgu 		import engine
from pgu 		import timer

# Local includes
import game
import gameparticles
import random
import globalvars

from data 			import *
from gamespaceobjs 	import *
from gameitems 		import *
from gameweapons 	import *
from gameships		import *
from stars 			import *
from globalvars		import *
from gameparticles 	import *



# Collision callback
def near_callback(args, geom1, geom2):
	# Callback function for the collide() method.
	# This function checks if the given geoms do collide and
	# creates contact joints if they do.
	
	# Check if the objects do collide
	contacts = ode.collide(geom1, geom2)
	
	# Create contact joints
	world,contactgroup = args
	for c in contacts:
		c.setBounce(0.1)
		c.setMu(500)
		j = ode.ContactJoint(world, contactgroup, c)
		j.attach(geom1.getBody(), geom2.getBody())

		contactPos = c.getContactGeomParams()[0]
		geom1.owner.OnCollide(geom2.owner, contactPos)
		geom2.owner.OnCollide(geom1.owner, contactPos)


class Game(engine.State):
	
	WRAP_SIZE 		= (screenWidth, screenHeight) # This is the radius that objects in space wrap around the player
	geomsToRemove 	= [] # A list to mark geoms for removal from the geom space
	cameraFocus 	= (0,0,0)
	guiFnt			= None
	
	guiCashGfk		= None
	guiOreGfk		= None
	guiMineralsGfk	= None
	guiWaterGfk		= None
	guiFuelGfk		= None
	guiHealthGfk	= None
	asteroidField 	= None
	
	randomizedweapons = [ BluePlasmaCannon, GreenPlasmaCannon, BaseCannonI, BaseCannonIII, SpitfireCannon, SingularityProvider ]
	#randomizedweapons = [ BluePlasmaCannon(self.playerShip), GreenPlasmaCannon(self.playerShip), BaseCannonI(self.playerShip), BaseCannonIII(self.playerShip), SpitfireCannon(self.playerShip), SingularityProvider(self.playerShip) ]
	
	def __init__(self, game, asteroidField):
		self.game = game
		self.asteroidField = asteroidField
		
	def init(self):
		if self.game.sound:
			self.clickSound = loadSound("button-click.ogg")
			if self.game.musicPlaying != "asteroids":
				pygame.mixer.music.fadeout(500)
				
				loadMusic("song10.ogg")
				pygame.mixer.music.play(-1)
				
				self.game.musicPlaying = "asteroids"
		else:
			self.clickSound = None


		self.guiFnt	= pygame.font.Font(os.path.join("data","lcd-dot-matrix.ttf"),20)	
		self.guiCashGfk		= pygame.transform.scale(loadImage("resource_cash_1.png"), (20,20) )
		self.guiOreGfk		= pygame.transform.scale(loadImage("resource_ore_1.png"), (20,20) )
		self.guiMineralsGfk	= pygame.transform.scale(loadImage("resource_minerals_1.png"), (20,20) )
		self.guiWaterGfk	= pygame.transform.scale(loadImage("resource_water_1.png"), (20,20) )
		self.guiFuelGfk		= pygame.transform.scale(loadImage("resource_fuel_1.png"), (20,20) )
		self.guiHealthGfk	= pygame.transform.scale(loadImage("resource_health_1.png"), (20,20) )

		
		# Create a dictionary to hold our key states
		self.keys = {}

		self.paused = False
		
		# Create a world object
		self.worldObj = ode.World()
		self.spaceObj = ode.Space()
		# A joint group for the contact joints that are generated whenever two bodies collide
		self.contactGroup = ode.JointGroup()
		
		
		# Create our sprite groups
		self.starGroup 		= pygame.sprite.RenderPlain()
		self.shipGroup 		= pygame.sprite.RenderPlain()
		self.asteroidGroup 	= pygame.sprite.RenderPlain()
		self.bulletGroup 	= pygame.sprite.RenderPlain()
		self.particleGroup 	= pygame.sprite.RenderPlain()
				
		self.last_loop_time = gameTime()
		
		
		# Create the star field
		self.stars = StarBackground(self)
		
		#For new games, and to set up a ship as a failsafe
		#Create ship
		self.playerShip = player.shipType(self, (0,0,0))
		self.playerShip.playerControlled = True
		for item in player.equipment:
			self.playerShip.equipment.append( item(self.playerShip) )

#		self.playerShip.equipment.append( MainThruster(self.playerShip) )
#		self.playerShip.equipment.append( BowThruster(self.playerShip) )
#		self.playerShip.equipment.append( AuxThruster(self.playerShip) )
		self.playerShip.equipment.append( self.randomizedweapons[random.randrange(6)](self.playerShip) )
		
		#self.background = loadImage("space_backdrop1.png")

		self.spawnAsteroids()

		self.cameraFocus = self.playerShip.pos
		
		self.backButton = game.Button(loadImage("back_button.png"), [10, 550], True)

	def createEffect(self, Effect, pos):
#		print "Creating new effect of type " + str(Effect) + " at " + str(pos)

		for inx in range(len(Effect.particles)):
#			print " Creating particle of type " + str(Effect.particles[inx])
			for cnt in range(Effect.quantities[inx]):
#				print "  Creating particle #" + str(cnt)
				
				part = Effect.particles[inx](self, pos) # Create a new particle
				
				# Set the parameters of the particle
				
				part.lifetime = Effect.lifetimes[inx] + random.random() * Effect.lifeVariance[inx]
				firingAngle = Effect.angle[inx] * 57.3 + random.random() * Effect.angleVariance[inx] + Effect.overallAngle
				velocity = Effect.velocities[inx] + random.random() * Effect.velVariance[inx]
				
				xvel = velocity * math.cos(firingAngle)
				yvel = velocity * math.sin(firingAngle)
				
				part.bodyObj.setPosition(pos)
				part.bodyObj.setLinearVel((xvel, yvel, 0))
			

	def spawnAsteroids(self):
		field = self.asteroidField
		
		if (field != None):
			for size in range(len(field.initRockSizes)):
				for numRocks in range(field.initRockSizes[size]):
					self.makeAsteroid(RockAsteroid, size)
				for numRocks in range(field.initMineralSizes[size]):
					self.makeAsteroid(MineralAsteroid, size)
				for numRocks in range(field.initOreSizes[size]):
					self.makeAsteroid(OreAsteroid, size)
				for numRocks in range(field.initWaterSizes[size]):
					self.makeAsteroid(IceAsteroid, size)
				

	def makeAsteroid(self, astType, size):
		rock_x = random.randrange(-screenWidth, screenWidth)
		rock_y = random.randrange(-screenHeight, screenHeight)

		newAsteroid = astType(self, (rock_x, rock_y, 0), size)

		firingAngle = random.random() * 6.3 # (2 * pi)		
		velocity = self.asteroidField.avgSpeed * 2 * random.random()
		
		xvel = velocity * math.cos(firingAngle)
		yvel = velocity * math.sin(firingAngle)
		
		newAsteroid.bodyObj.setPosition((rock_x, rock_y, 0))
		newAsteroid.bodyObj.setLinearVel((xvel, yvel, 0))
	
		rotVel = self.asteroidField.avgSpin * 2 * random.random() - self.asteroidField.avgSpin

		newAsteroid.bodyObj.setAngularVel((0,0,rotVel))


	def spawnRandomAsteroids(self, numRocks):
		for cnt in range(0, numRocks):
			rock_x = random.randrange(-screenWidth, screenWidth)
			rock_y = random.randrange(-screenHeight, screenHeight)
			asteroidClasses = [RockAsteroid, MineralAsteroid, IceAsteroid, OreAsteroid]
			randomAsteroid = asteroidClasses[random.randrange(4)]
			
			newRock = randomAsteroid(self, (rock_x, rock_y, 0), random.randrange(4) + 3)

	def loop(self):
		if self.backButton.release:
			if self.clickSound != None:
				self.clickSound.play(0)
			return game.NavUI(self.game)
		
		current_time = gameTime()

		if not self.paused:
			# Delta_t is the amount of time that this frame needs to process for 
			delta_t = current_time - self.last_loop_time

			# Move the camera -- update the player ship first so that we can have an accurate camera location for the other sprites (this removes camera jitter)
			self.playerShip.update(0)
			self.cameraFocus = (self.playerShip.pos[0] - screenWidth / 2, self.playerShip.pos[1] - screenHeight / 2) 


			self.starGroup.update(delta_t)

			self.shipGroup.update(delta_t)
			self.asteroidGroup.update(delta_t)
			self.bulletGroup.update(delta_t)
			
			self.particleGroup.update(delta_t)

			
			# Do collisions
			# Detect collisions and create contact joints
			self.spaceObj.collide ( (self.worldObj, self.contactGroup), near_callback)
			
			# Remove any geoms that are marked for termination
			for geom in self.geomsToRemove:
				if (self.spaceObj.query(geom)):
					self.spaceObj.remove(geom)
			
			self.geomsToRemove = []
			
			# Update the physics of the world
			self.worldObj.step(delta_t)

			# Clear the old contact joints		
			self.contactGroup.empty()
		
		self.last_loop_time = current_time



	def update(self,screen):	
		return self.paint(screen)

	def paint(self,screen):
		# Draw the graphics, starting with the star background (which is opaque, and we don't need to clear the screen before drawing it), all the way up to particle effects.
		self.starGroup.draw(screen)
		self.asteroidGroup.draw(screen)
		self.bulletGroup.draw(screen)
		self.shipGroup.draw(screen)
		
		self.particleGroup.draw(screen)
		
		# TODO: We shouldn't be loading this font from the file *every* frame
		tempInstructions = self.guiFnt.render("Press the arrow keys to move the ship!",1,(255,255,255))
		x2 = 150
		y2 = 575
		screen.blit(tempInstructions,(x2,y2))
		
		cashText 		= self.guiFnt.render( str(player.cash), 			1, (255,255,255) )
		oreText 		= self.guiFnt.render( str(player.getOre()), 		1, (255,255,255) )
		mineralsText 	= self.guiFnt.render( str(player.getMineral()), 	1, (255,255,255) )
		waterText		= self.guiFnt.render( str(player.getWater()), 		1, (255,255,255) )
		fuelText 		= self.guiFnt.render( str(player.getFuel()), 		1, (255,255,255) )
		
		health			= self.playerShip.hitpoints * 1.0 / player.shipType.hitpoints
		if (health < 0):
			health = 0
		
		if (health > 1):
			health = 1
			
		healthColor		= (int(255 * (1 - health)), 100, int(255 * health))
		healthPct		= int(health * 100)
		
		healthText 		= self.guiFnt.render( str(healthPct) + "%", 			1, healthColor )

		
		screen.blit(self.guiCashGfk, 		(10, 10))
		screen.blit(self.guiOreGfk, 		(10, 30))
		screen.blit(self.guiMineralsGfk, 	(10, 50))
		screen.blit(self.guiWaterGfk, 		(10, 70))
		screen.blit(self.guiFuelGfk, 		(10, 90))
		screen.blit(self.guiHealthGfk, 		(10, 110))
		
		screen.blit(cashText, 		(30, 15))
		screen.blit(oreText, 		(30, 35))
		screen.blit(mineralsText, 	(30, 55))
		screen.blit(waterText, 		(30, 75))
		screen.blit(fuelText, 		(30, 95))
		screen.blit(healthText, 	(30, 115))
		
		
		if (self.playerShip.hitpoints <= 0):
			deadText = self.guiFnt.render("You have lost all of your cargo and been ejected to space.", 1, (255, 0, 0))
			deadText2 = self.guiFnt.render("Press ESC or Back to return to nav map and try again.", 1, (255, 0, 0))
			screen.blit(deadText,	(10, 135))
			screen.blit(deadText2, 	(10, 155))
			
			player.setOre(0)
			player.setMineral(0)
			player.setWater(0)
			player.setFuel(0)
			
		elif (player.getHullSpaceLeft() <= 0):
			fullText = self.guiFnt.render("CARGO BAY FULL", 1, (255, 0, 0))
			screen.blit(fullText,	(10, 135))
		
		self.backButton.update(screen, pygame.mouse.get_pos(),  pygame.mouse.get_pressed()[0])
			
		pygame.display.update()

	def event(self,e):
		if e.type == KEYDOWN:
			self.keys[e.key] = True

			if e.key == K_ESCAPE:
				return game.NavUI(self.game)
#				import menu
#				return menu.Menu(self.game)
			elif e.key == K_n:
				pass
				
#				self.newShip(100, (1,1,0))
#				for ship in self.ships:
#					if self.ships.index(ship) > 0:
#						ship[0].addForce((-3,0,0))

			elif e.key == K_s:
				pass

			elif e.key in (K_p,K_PAUSE):
				self.paused = not self.paused

		elif e.type == KEYUP:
			self.keys[e.key] = False

