import pygame
import random

from globalvars		import *


from pygame.locals import *

class Star:
	basePos = pos = (0,0)
	distance = 1

class StarBackground(pygame.sprite.Sprite):
	numStars 	= 500
	stars 		= []
	
	def __init__(self, gameObj):
		self.gameObj = gameObj
		
		# GRAPHICS
		pygame.sprite.Sprite.__init__(self) #call Sprite initializer
		
		# Create our blank image
		self.image = pygame.Surface((screenWidth, screenHeight))
		self.rect = self.image.get_rect()
		
		self.gameObj.starGroup.add(self)
		
		self.initializeStars()
		
		
	def initializeStars(self):
		for cnt in range(self.numStars):
			s = Star()
			s.basePos = (random.randrange(0, screenWidth), random.randrange(0, screenHeight))

			# Get a distribution with few in the back, most in the middle, and not many at the front
			s.distance = int(1.13 * random.random() * 6 * random.random() * 6 * random.random() * 5 + 51.6)

			self.stars.append( s )

	def update(self, delta_t):
		self.draw_stars()
	
	def draw_stars(self):
		cameraPos = (self.gameObj.cameraFocus[0] - screenWidth / 2, self.gameObj.cameraFocus[1] - screenHeight / 2)

		# Clear the background
		self.image.fill((0,0,0))

		# Update the stars
		for star in self.stars:
#			x = star.basePos[0] - cameraPos[0] * (star.distance / 200.0) / 1000.0
#			x = (x + screenWidth) % screenWidth
#			y = star.basePos[1] - cameraPos[1] * (star.distance / 200.0) / 1000.0
#			y = (y + screenHeight) % screenHeight
			x = ( screenWidth  + star.basePos[0] - ( cameraPos[0] * star.distance / 400.0 ) ) % screenWidth 
			y = ( screenHeight + star.basePos[1] - ( cameraPos[1] * star.distance / 400.0 ) ) % screenHeight 
			star.pos = (int(x),int(y))

		# Draw the stars
		self.image.lock
				
		for star in self.stars:
			self.image.set_at(star.pos, (star.distance, star.distance, star.distance))
#			self.set_at_aa(self.image, star.pos, star.distance)
		
		self.image.unlock

		
	def set_at_aa(self, surf, pos, brightness):
		# Loop between both X columns
		for xcnt in range(2):
			## Check for clipping in the X direction (make sure it fits horizontally)
			#if ((int)p.X - xcnt >= 0 && (int)p.X - xcnt < s.Width)
			# Loop between both Y rows
			for ycnt in range(2):
				## Check for clipping in the Y direction (make sure it fits vertically)
				#if ((int)p.Y - ycnt >= 0 && (int)p.Y - ycnt < s.Height)
			
				# Get the x component of the brightness and divide it by 2
				xcomponent = abs((xcnt - (pos[0] % 1)) / 2)
				# Get the y component of the brightness and divide it by 2
				ycomponent = abs((ycnt - (pos[1] % 1)) / 2)
				
				# Add the x and y components of the brightness to get the total brightness at that particular location
 				drawBrightness = int(xcomponent + ycomponent) * brightness

				# Derive the draw color from the brightness
				drawColor = (drawBrightness, drawBrightness, drawBrightness)
				
				# Only draw the pixel if it'll make that point get brighter -- we don't want to draw stars that are further behind (darker) in front of nearer (brighter) stars
				if (surf.get_at(pos)[1] < drawBrightness):
					surf.set_at((pos[0] - xcnt, pos[1] - ycnt), drawColor)
